'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _utils = require('./utils');

var utils = _interopRequireWildcard(_utils);

var Interpolator = (function () {
  function Interpolator() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, Interpolator);

    this.init(options, true);
  }

  _createClass(Interpolator, [{
    key: 'init',
    value: function init(options, reset) {
      if (options === undefined) options = {};

      if (reset) this.options = options;
      if (!options.interpolation) options.interpolation = { escapeValue: true };

      this.escapeValue = options.interpolation.escapeValue;

      this.prefix = options.interpolation.prefix ? utils.regexEscape(options.interpolation.prefix) : options.interpolation.prefixEscaped || '{{';
      this.suffix = options.interpolation.suffix ? utils.regexEscape(options.interpolation.suffix) : options.interpolation.suffixEscaped || '}}';

      this.unescapePrefix = options.interpolation.unescapeSuffix ? '' : options.interpolation.unescapePrefix || '-';
      this.unescapeSuffix = this.unescapePrefix ? '' : options.interpolation.unescapeSuffix || '';

      this.nestingPrefix = options.interpolation.nestingPrefix ? utils.regexEscape(options.interpolation.nestingPrefix) : options.interpolation.nestingPrefixEscaped || '$t(';
      this.nestingSuffix = options.interpolation.nestingSuffix ? utils.regexEscape(options.interpolation.nestingSuffix) : options.interpolation.nestingSuffixEscaped || ')';

      // the regexp
      var regexpStr = this.prefix + '(.+?)' + this.suffix;
      this.regexp = new RegExp(regexpStr, 'g');

      var regexpUnescapeStr = this.prefix + this.unescapePrefix + '(.+?)' + this.unescapeSuffix + this.suffix;
      this.regexpUnescape = new RegExp(regexpUnescapeStr, 'g');

      var nestingRegexpStr = this.nestingPrefix + '(.+?)' + this.nestingSuffix;
      this.nestingRegexp = new RegExp(nestingRegexpStr, 'g');
    }
  }, {
    key: 'reset',
    value: function reset() {
      if (this.options) this.init(this.options);
    }
  }, {
    key: 'interpolate',
    value: function interpolate(str, data) {
      var match = undefined,
          value = undefined;

      function regexSafe(val) {
        return val.replace(/\$/g, '$$$$');
      }

      // unescape if has unescapePrefix/Suffix
      while (match = this.regexpUnescape.exec(str)) {
        var _value = utils.getPath(data, match[1].trim());
        str = str.replace(match[0], _value);
      }

      // regular escape on demand
      while (match = this.regexp.exec(str)) {
        value = utils.getPath(data, match[1].trim());
        if (value !== undefined && typeof value !== 'string') value = value.toString(); // TODO: warn for values matching pattern even if no risk for xss
        if (!value) value = ''; // TODO: warn for missing value
        value = this.escapeValue ? regexSafe(utils.escape(value)) : regexSafe(value);
        str = str.replace(match[0], value);
        this.regexp.lastIndex = 0;
      }
      return str;
    }
  }, {
    key: 'nest',
    value: function nest(str, fc, options) {
      var match = undefined,
          value = undefined;

      function regexSafe(val) {
        return val.replace(/\$/g, '$$$$');
      }

      // if value is something like "myKey": "lorem $(anotherKey, { "count": {{aValueInOptions}} })"
      function handleHasOptions(key) {
        if (key.indexOf(',') < 0) return key;

        var p = key.split(',');
        key = p.shift();
        var optionsString = p.join(',');
        optionsString = this.interpolate(optionsString, options);

        try {
          options = JSON.parse(optionsString);
        } catch (e) {
          // TODO: log error
        }

        return key;
      }

      // regular escape on demand
      while (match = this.nestingRegexp.exec(str)) {
        value = fc(handleHasOptions.call(this, match[1].trim()), options);
        if (value !== undefined && typeof value !== 'string') value = value.toString(); // TODO: warn for values matching pattern even if no risk for xss
        if (!value) value = ''; // TODO: warn for missing value
        value = this.escapeValue ? regexSafe(utils.escape(value)) : regexSafe(value);
        str = str.replace(match[0], value);
        this.regexp.lastIndex = 0;
      }
      return str;
    }
  }]);

  return Interpolator;
})();

exports['default'] = Interpolator;
module.exports = exports['default'];